package javax.swing.text;

import java.awt.Shape;
import java.awt.Rectangle;

public abstract class CompositeView extends View
{
    private static View[] ZERO;
    private View[] children;
    private int nchildren;
    private short left;
    private short right;
    private short top;
    private short bottom;
    private Rectangle childAlloc;
    
    public CompositeView(Element element) {
        super(element);
        this.children = new View[1];
        this.nchildren = 0;
        this.childAlloc = new Rectangle();
    }
    
    protected void loadChildren(ViewFactory viewFactory) {
        if (viewFactory == null) {
            return;
        }
        Element element = this.getElement();
        int elementCount = element.getElementCount();
        if (elementCount > 0) {
            View[] array = new View[elementCount];
            for (int i = 0; i < elementCount; ++i) {
                array[i] = viewFactory.create(element.getElement(i));
            }
            this.replace(0, 0, array);
        }
        return;
    }
    
    @Override
    public void setParent(View parent) {
        super.setParent(parent);
        if (parent != null && this.nchildren == 0) {
            this.loadChildren(this.getViewFactory());
        }
    }
    
    @Override
    public int getViewCount() {
        return this.nchildren;
    }
    
    @Override
    public View getView(int n) {
        return this.children[n];
    }
    
    @Override
    public void replace(int i, int n, View[] zero) {
        if (zero == null) {
            zero = CompositeView.ZERO;
        }
        for (; i < i + n; this.children[i] = null, ++i) {
            if (this.children[i].getParent() == this) {
                this.children[i].setParent(null);
            }
        }
        int n2 = zero.length - n;
        int n3 = i + n;
        int n4 = this.nchildren - n3;
        int n5 = n3 + n2;
        if (this.nchildren + n2 >= this.children.length) {
            View[] children = new View[Math.max(2 * this.children.length, this.nchildren + n2)];
            System.arraycopy(this.children, 0, children, 0, i);
            System.arraycopy(zero, 0, children, i, zero.length);
            System.arraycopy(this.children, n3, children, n5, n4);
            this.children = children;
        }
        else {
            System.arraycopy(this.children, n3, this.children, n5, n4);
            System.arraycopy(zero, 0, this.children, i, zero.length);
        }
        this.nchildren += n2;
        for (int j = 0; j < zero.length; ++j) {
            zero[j].setParent(this);
        }
    }
    
    @Override
    public Shape getChildAllocation(int n, Shape shape) {
        Rectangle insideAllocation = this.getInsideAllocation(shape);
        this.childAllocation(n, insideAllocation);
        return insideAllocation;
    }
    
    @Override
    public Shape modelToView(int n, Shape shape, Position.Bias bias) throws BadLocationException {
        int n2 = bias == Position.Bias.Backward;
        n = (n2 ? Math.max(0, n - 1) : n);
        if (n2 && n < this.getStartOffset()) {
            return null;
        }
        int viewIndexAtPosition = this.getViewIndexAtPosition(n);
        if (viewIndexAtPosition != -1 && viewIndexAtPosition < this.getViewCount()) {
            View view = this.getView(viewIndexAtPosition);
            if (view != null && n >= view.getStartOffset() && n < view.getEndOffset()) {
                Shape childAllocation = this.getChildAllocation(viewIndexAtPosition, shape);
                if (childAllocation == null) {
                    return null;
                }
                Shape shape2 = view.modelToView(n, childAllocation, bias);
                if (shape2 == null && view.getEndOffset() == n && ++viewIndexAtPosition < this.getViewCount()) {
                    shape2 = this.getView(viewIndexAtPosition).modelToView(n, this.getChildAllocation(viewIndexAtPosition, shape), bias);
                    return shape2;
                }
                return shape2;
            }
        }
        throw new BadLocationException("Position not represented by view", n);
    }
    
    @Override
    public Shape modelToView(int n, Position.Bias bias, int n2, Position.Bias bias2, Shape shape) throws BadLocationException {
        if (n == this.getStartOffset() && n2 == this.getEndOffset()) {
            return shape;
        }
        Rectangle insideAllocation = this.getInsideAllocation(shape);
        Rectangle rectangle = new Rectangle(insideAllocation);
        View viewAtPosition = this.getViewAtPosition((bias == Position.Bias.Backward) ? Math.max(0, n - 1) : n, rectangle);
        Rectangle rectangle2 = new Rectangle(insideAllocation);
        View viewAtPosition2 = this.getViewAtPosition((bias2 == Position.Bias.Backward) ? Math.max(0, n2 - 1) : n2, rectangle2);
        if (viewAtPosition == viewAtPosition2) {
            if (viewAtPosition == null) {
                return shape;
            }
            return viewAtPosition.modelToView(n, bias, n2, bias2, rectangle);
        }
        else {
            int viewCount = this.getViewCount();
            int i = 0;
            while (i < viewCount) {
                View view;
                if ((view = this.getView(i)) == viewAtPosition || view == viewAtPosition2) {
                    Rectangle r = new Rectangle();
                    Rectangle rectangle3;
                    if (view == viewAtPosition) {
                        rectangle3 = viewAtPosition.modelToView(n, bias, viewAtPosition.getEndOffset(), Position.Bias.Backward, rectangle).getBounds();
                    }
                    else {
                        rectangle3 = viewAtPosition2.modelToView(viewAtPosition2.getStartOffset(), Position.Bias.Forward, n2, bias2, rectangle2).getBounds();
                        viewAtPosition2 = viewAtPosition;
                    }
                    View view2;
                    while (++i < viewCount && (view2 = this.getView(i)) != viewAtPosition2) {
                        r.setBounds(insideAllocation);
                        this.childAllocation(i, r);
                        rectangle3.add(r);
                    }
                    if (viewAtPosition2 != null) {
                        Shape shape2;
                        if (viewAtPosition2 == viewAtPosition2) {
                            shape2 = viewAtPosition2.modelToView(viewAtPosition2.getStartOffset(), Position.Bias.Forward, n2, bias2, rectangle2);
                        }
                        else {
                            shape2 = viewAtPosition.modelToView(n, bias, viewAtPosition.getEndOffset(), Position.Bias.Backward, rectangle);
                        }
                        if ((shape2 instanceof Rectangle)) {
                            rectangle3.add((Rectangle)shape2);
                        }
                        else {
                            rectangle3.add(shape2.getBounds());
                        }
                    }
                    return rectangle3;
                }
                else {
                    ++i;
                }
            }
            throw new BadLocationException("Position not represented by view", n);
        }
    }
    
    @Override
    public int viewToModel(float n, float n2, Shape shape, Position.Bias[] array) {
        Rectangle insideAllocation = this.getInsideAllocation(shape);
        if (this.isBefore((int)n, (int)n2, insideAllocation)) {
            int n3 = -1;
            try {
                n3 = this.getNextVisualPositionFrom(-1, Position.Bias.Forward, shape, 3, array);
            }
            catch (BadLocationException ex3) {}
            catch (IllegalArgumentException ex4) {}
            if (n3 == -1) {
                n3 = this.getStartOffset();
                array[0] = Position.Bias.Forward;
            }
            return n3;
        }
        else if (this.isAfter((int)n, (int)n2, insideAllocation)) {
            int nextVisualPosition = -1;
            try {
                nextVisualPosition = this.getNextVisualPositionFrom(-1, Position.Bias.Forward, shape, 7, array);
            }
            catch (BadLocationException ex) {}
            catch (IllegalArgumentException ex2) {}
            if (nextVisualPosition == -1) {
                nextVisualPosition = this.getEndOffset() - 1;
                array[0] = Position.Bias.Forward;
            }
            return nextVisualPosition;
        }
        else {
            View viewAtPoint = this.getViewAtPoint((int)n, (int)n2, insideAllocation);
            if (viewAtPoint != null) {
                return viewAtPoint.viewToModel(n, n2, insideAllocation, array);
            }
            return -1;
        }
    }
    
    @Override
    public int getNextVisualPositionFrom(int n, Position.Bias bias, Shape shape, int i, Position.Bias[] array) throws BadLocationException {
        if (n < -1) {
            throw new BadLocationException("invalid position", n);
        }
        this.getInsideAllocation(shape);
        switch (i) {
            case 1: {
                return this.getNextNorthSouthVisualPositionFrom(n, bias, shape, i, array);
            }
            case 5: {
                return this.getNextNorthSouthVisualPositionFrom(n, bias, shape, i, array);
            }
            case 3: {
                return this.getNextEastWestVisualPositionFrom(n, bias, shape, i, array);
            }
            case 7: {
                return this.getNextEastWestVisualPositionFrom(n, bias, shape, i, array);
            }
            default: {
                throw new IllegalArgumentException("Bad direction: " + i);
            }
        }
    }
    
    @Override
    public int getViewIndex(int n, Position.Bias bias) {
        if (bias == Position.Bias.Backward) {
            --n;
        }
        if (n >= this.getStartOffset() && n < this.getEndOffset()) {
            return this.getViewIndexAtPosition(n);
        }
        return -1;
    }
    
    protected abstract boolean isBefore(int p0, int p1, Rectangle p2);
    
    protected abstract boolean isAfter(int p0, int p1, Rectangle p2);
    
    protected abstract View getViewAtPoint(int p0, int p1, Rectangle p2);
    
    protected abstract void childAllocation(int p0, Rectangle p1);
    
    protected View getViewAtPosition(int n, Rectangle rectangle) {
        int viewIndexAtPosition = this.getViewIndexAtPosition(n);
        if (viewIndexAtPosition >= 0 && viewIndexAtPosition < this.getViewCount()) {
            View view = this.getView(viewIndexAtPosition);
            if (rectangle != null) {
                this.childAllocation(viewIndexAtPosition, rectangle);
                return view;
            }
            return view;
        }
        else {
            return null;
        }
    }
    
    protected int getViewIndexAtPosition(int n) {
        return this.getElement().getElementIndex(n);
    }
    
    protected Rectangle getInsideAllocation(Shape shape) {
        if (shape != null) {
            Rectangle bounds;
            if ((shape instanceof Rectangle)) {
                bounds = (Rectangle)shape;
            }
            else {
                bounds = shape.getBounds();
            }
            this.childAlloc.setBounds(bounds);
            Rectangle childAlloc = this.childAlloc;
            childAlloc.x += this.getLeftInset();
            Rectangle childAlloc2 = this.childAlloc;
            childAlloc2.y += this.getTopInset();
            Rectangle childAlloc3 = this.childAlloc;
            childAlloc3.width -= this.getLeftInset() + this.getRightInset();
            Rectangle childAlloc4 = this.childAlloc;
            childAlloc4.height -= this.getTopInset() + this.getBottomInset();
            return this.childAlloc;
        }
        else {
            return null;
        }
    }
    
    protected void setParagraphInsets(AttributeSet set) {
        this.top = (short)(int)StyleConstants.getSpaceAbove(set);
        this.left = (short)(int)StyleConstants.getLeftIndent(set);
        this.bottom = (short)(int)StyleConstants.getSpaceBelow(set);
        this.right = (short)(int)StyleConstants.getRightIndent(set);
    }
    
    protected void setInsets(short top, short left, short bottom, short right) {
        this.top = top;
        this.left = left;
        this.right = right;
        this.bottom = bottom;
    }
    
    protected short getLeftInset() {
        return this.left;
    }
    
    protected short getRightInset() {
        return this.right;
    }
    
    protected short getTopInset() {
        return this.top;
    }
    
    protected short getBottomInset() {
        return this.bottom;
    }
    
    protected int getNextNorthSouthVisualPositionFrom(int n, Position.Bias bias, Shape shape, int n2, Position.Bias[] array) throws BadLocationException {
        return Utilities.getNextVisualPositionFrom(this, n, bias, shape, n2, array);
    }
    
    protected int getNextEastWestVisualPositionFrom(int n, Position.Bias bias, Shape shape, int n2, Position.Bias[] array) throws BadLocationException {
        return Utilities.getNextVisualPositionFrom(this, n, bias, shape, n2, array);
    }
    
    protected boolean flipEastAndWestAtEnds(int n, Position.Bias bias) {
        return false;
    }
    
    static {
        CompositeView.ZERO = new View[0];
    }
}
